@@ -34,7 +34,7 @@ $(function() {
 		$("#ok-button", $actionsDialog).attr("disabled","disabled");
 
 		// Submit a POST request to find action
-		window.sendMessage("find-action", { "action": action }, function(matches) {
+		$.post("/find_action", { "action": action }, function(matches) {
 				
 			// process matches and make select list html
 			var html = '';
