@@ -1,4 +1,15 @@
-CodeMirror.validate = (function() {
+// CodeMirror, copyright (c) by Marijn Haverbeke and others
+// Distributed under an MIT license: http://codemirror.net/LICENSE
+
+(function(mod) {
+  if (typeof exports == "object" && typeof module == "object") // CommonJS
+    mod(require("../../lib/codemirror"));
+  else if (typeof define == "function" && define.amd) // AMD
+    define(["../../lib/codemirror"], mod);
+  else // Plain browser env
+    mod(CodeMirror);
+})(function(CodeMirror) {
+  "use strict";
   var GUTTER_ID = "CodeMirror-lint-markers";
   var SEVERITIES = /^(?:error|warning)$/;
 
@@ -10,12 +21,12 @@ CodeMirror.validate = (function() {
 
     function position(e) {
       if (!tt.parentNode) return CodeMirror.off(document, "mousemove", position);
-      tt.style.top = (e.clientY - tt.offsetHeight - 5) + "px";
+      tt.style.top = Math.max(0, e.clientY - tt.offsetHeight - 5) + "px";
       tt.style.left = (e.clientX + 5) + "px";
     }
     CodeMirror.on(document, "mousemove", position);
     position(e);
-    tt.style.opacity = 1;
+    if (tt.style.opacity != null) tt.style.opacity = 1;
     return tt;
   }
   function rm(elt) {
@@ -28,6 +39,22 @@ CodeMirror.validate = (function() {
     setTimeout(function() { rm(tt); }, 600);
   }
 
+  function showTooltipFor(e, content, node) {
+    var tooltip = showTooltip(e, content);
+    function hide() {
+      CodeMirror.off(node, "mouseout", hide);
+      if (tooltip) { hideTooltip(tooltip); tooltip = null; }
+    }
+    var poll = setInterval(function() {
+      if (tooltip) for (var n = node;; n = n.parentNode) {
+        if (n == document.body) return;
+        if (!n) { hide(); break; }
+      }
+      if (!tooltip) return clearInterval(poll);
+    }, 400);
+    CodeMirror.on(node, "mouseout", hide);
+  }
+
   function LintState(cm, options, hasGutter) {
     this.marked = [];
     this.options = options;
@@ -36,21 +63,23 @@ CodeMirror.validate = (function() {
     this.onMouseOver = function(e) { onMouseOver(cm, e); };
   }
 
-  function parseOptions(options) {
+  function parseOptions(cm, options) {
     if (options instanceof Function) return {getAnnotations: options};
-    else if (!options || !options.getAnnotations) throw new Error("Required option 'getAnnotations' missing (lint addon)");
+    if (!options || options === true) options = {};
+    if (!options.getAnnotations) options.getAnnotations = cm.getHelper(CodeMirror.Pos(0, 0), "lint");
+    if (!options.getAnnotations) throw new Error("Required option 'getAnnotations' missing (lint addon)");
     return options;
   }
 
   function clearMarks(cm) {
-    var state = cm._lintState;
+    var state = cm.state.lint;
     if (state.hasGutter) cm.clearGutter(GUTTER_ID);
     for (var i = 0; i < state.marked.length; ++i)
       state.marked[i].clear();
     state.marked.length = 0;
   }
 
-  function makeMarker(labels, severity, multiple) {
+  function makeMarker(labels, severity, multiple, tooltips) {
     var marker = document.createElement("div"), inner = marker;
     marker.className = "CodeMirror-lint-marker-" + severity;
     if (multiple) {
@@ -58,9 +87,9 @@ CodeMirror.validate = (function() {
       inner.className = "CodeMirror-lint-marker-multiple";
     }
 
-    var tooltip;
-    CodeMirror.on(inner, "mouseover", function(e) { tooltip = showTooltip(e, labels); });
-    CodeMirror.on(inner, "mouseout", function() { if (tooltip) hideTooltip(tooltip); });
+    if (tooltips != false) CodeMirror.on(inner, "mouseover", function(e) {
+      showTooltipFor(e, labels, inner);
+    });
 
     return marker;
   }
@@ -89,16 +118,16 @@ CodeMirror.validate = (function() {
   }
 
   function startLinting(cm) {
-	  var state = cm._lintState, options = state.options;
-	  if (options.async)
-		  options.getAnnotations(cm, updateLinting, options);
-	  else
-		 updateLinting(cm, options.getAnnotations(cm.getValue()));
+    var state = cm.state.lint, options = state.options;
+    if (options.async)
+      options.getAnnotations(cm, updateLinting, options);
+    else
+      updateLinting(cm, options.getAnnotations(cm.getValue(), options.options));
   }
-  
+
   function updateLinting(cm, annotationsNotSorted) {
     clearMarks(cm);
-    var state = cm._lintState, options = state.options;
+    var state = cm.state.lint, options = state.options;
 
     var annotations = groupByLine(annotationsNotSorted);
 
@@ -115,35 +144,31 @@ CodeMirror.validate = (function() {
         if (!SEVERITIES.test(severity)) severity = "error";
         maxSeverity = getMaxSeverity(maxSeverity, severity);
 
-	if (options.formatAnnotation) ann = options.formatAnnotation(ann);
+        if (options.formatAnnotation) ann = options.formatAnnotation(ann);
         if (state.hasGutter) tipLabel.appendChild(annotationTooltip(ann));
 
         if (ann.to) state.marked.push(cm.markText(ann.from, ann.to, {
-          className: "CodeMirror-lint-span-" + severity,
+          className: "CodeMirror-lint-mark-" + severity,
           __annotation: ann
         }));
       }
 
       if (state.hasGutter)
-        cm.setGutterMarker(line, GUTTER_ID, makeMarker(tipLabel, maxSeverity, anns.length > 1));
+        cm.setGutterMarker(line, GUTTER_ID, makeMarker(tipLabel, maxSeverity, anns.length > 1,
+                                                       state.options.tooltips));
     }
     if (options.onUpdateLinting) options.onUpdateLinting(annotationsNotSorted, annotations, cm);
   }
 
   function onChange(cm) {
-    var state = cm._lintState;
+    var state = cm.state.lint;
     clearTimeout(state.timeout);
     state.timeout = setTimeout(function(){startLinting(cm);}, state.options.delay || 500);
   }
 
   function popupSpanTooltip(ann, e) {
-    var tooltip = showTooltip(e, annotationTooltip(ann));
     var target = e.target || e.srcElement;
-    CodeMirror.on(target, "mouseout", hide);
-    function hide() {
-      CodeMirror.off(target, "mouseout", hide);
-      hideTooltip(tooltip);
-    }
+    showTooltipFor(e, annotationTooltip(ann), target);
   }
 
   // When the mouseover fires, the cursor might not actually be over
@@ -152,10 +177,10 @@ CodeMirror.validate = (function() {
   var nearby = [0, 0, 0, 5, 0, -5, 5, 0, -5, 0];
 
   function onMouseOver(cm, e) {
-    if (!/\bCodeMirror-lint-span-/.test((e.target || e.srcElement).className)) return;
+    if (!/\bCodeMirror-lint-mark-/.test((e.target || e.srcElement).className)) return;
     for (var i = 0; i < nearby.length; i += 2) {
       var spans = cm.findMarksAt(cm.coordsChar({left: e.clientX + nearby[i],
-                                                top: e.clientY + nearby[i + 1]}));
+                                                top: e.clientY + nearby[i + 1]}, "client"));
       for (var j = 0; j < spans.length; ++j) {
         var span = spans[j], ann = span.__annotation;
         if (ann) return popupSpanTooltip(ann, e);
@@ -163,21 +188,23 @@ CodeMirror.validate = (function() {
     }
   }
 
-  CodeMirror.defineOption("lintWith", false, function(cm, val, old) {
+  CodeMirror.defineOption("lint", false, function(cm, val, old) {
     if (old && old != CodeMirror.Init) {
       clearMarks(cm);
       cm.off("change", onChange);
-      CodeMirror.off(cm.getWrapperElement(), "mouseover", cm._lintState.onMouseOver);
-      delete cm._lintState;
+      CodeMirror.off(cm.getWrapperElement(), "mouseover", cm.state.lint.onMouseOver);
+      delete cm.state.lint;
     }
-    
+
     if (val) {
       var gutters = cm.getOption("gutters"), hasLintGutter = false;
       for (var i = 0; i < gutters.length; ++i) if (gutters[i] == GUTTER_ID) hasLintGutter = true;
-      var state = cm._lintState = new LintState(cm, parseOptions(val), hasLintGutter);
+      var state = cm.state.lint = new LintState(cm, parseOptions(cm, val), hasLintGutter);
       cm.on("change", onChange);
-      CodeMirror.on(cm.getWrapperElement(), "mouseover", state.onMouseOver);
+      if (state.options.tooltips != false)
+        CodeMirror.on(cm.getWrapperElement(), "mouseover", state.onMouseOver);
+
       startLinting(cm);
     }
   });
-})();
+});
