@@ -1,26 +1,28 @@
+// CodeMirror, copyright (c) by Marijn Haverbeke and others
+// Distributed under an MIT license: http://codemirror.net/LICENSE
+
+(function(mod) {
+  if (typeof exports == "object" && typeof module == "object") // CommonJS
+    mod(require("../../lib/codemirror"));
+  else if (typeof define == "function" && define.amd) // AMD
+    define(["../../lib/codemirror"], mod);
+  else // Plain browser env
+    mod(CodeMirror);
+})(function(CodeMirror) {
+"use strict";
+
 CodeMirror.defineMode("sass", function(config) {
   var tokenRegexp = function(words){
     return new RegExp("^" + words.join("|"));
   };
 
-  var tags = ["&", "a","abbr","acronym","address","applet","area","article","aside","audio","b","base","basefont","bdi","bdo","big","blockquote","body","br","button","canvas","caption","cite","code","col","colgroup","command","datalist","dd","del","details","dfn","dir","div","dl","dt","em","embed","fieldset","figcaption","figure","font","footer","form","frame","frameset","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","iframe","img","input","ins","keygen","kbd","label","legend","li","link","map","mark","menu","meta","meter","nav","noframes","noscript","object","ol","optgroup","option","output","p","param","pre","progress","q","rp","rt","ruby","s","samp","script","section","select","small","source","span","strike","strong","style","sub","summary","sup","table","tbody","td","textarea","tfoot","th","thead","time","title","tr","track","tt","u","ul","var","video","wbr"];
   var keywords = ["true", "false", "null", "auto"];
   var keywordsRegexp = new RegExp("^" + keywords.join("|"));
 
   var operators = ["\\(", "\\)", "=", ">", "<", "==", ">=", "<=", "\\+", "-", "\\!=", "/", "\\*", "%", "and", "or", "not"];
   var opRegexp = tokenRegexp(operators);
 
-  function htmlTag(val){
-    for(var i=0; i<tags.length; i++){
-      if(val === tags[i]){
-        return true;
-      }
-    }
-  }
-
-
-  var pseudoElements = [':first-line', ':hover', ':first-letter', ':active', ':visited', ':before', ':after', ':link', ':focus', ':first-child', ':lang'];
-  var pseudoElementsRegexp = new RegExp("^(" + pseudoElements.join("\\b|") + ")");
+  var pseudoElementsRegexp = /^::?[\w\-]+/;
 
   var urlTokens = function(stream, state){
     var ch = stream.peek();
@@ -256,25 +258,17 @@ CodeMirror.defineMode("sass", function(config) {
     }
 
     // Pseudo element selectors
-    if (stream.match(pseudoElementsRegexp)){
+    if (ch == ':' && stream.match(pseudoElementsRegexp)){
       return "keyword";
     }
 
     // atoms
     if (stream.eatWhile(/[\w-&]/)){
-
-      var current = stream.current();
       // matches a property definition
-      if (stream.peek() === ":"){
-        // if this is an html tag and it has a pseudo selector, then it's an atom
-        if (htmlTag(current) && stream.match(pseudoElementsRegexp, false)){
-          return "atom";
-        }else{
-          stream.next();
-          return "property";
-        }
-      }
-      return "atom";
+      if (stream.peek() === ":" && !stream.match(pseudoElementsRegexp, false))
+        return "property";
+      else
+        return "atom";
     }
 
     if (stream.match(opRegexp)){
@@ -284,7 +278,7 @@ CodeMirror.defineMode("sass", function(config) {
     // If we haven't returned by now, we move 1 character
     // and return an error
     stream.next();
-    return 'error';
+    return null;
   };
 
   var tokenLexer = function(stream, state) {
@@ -298,11 +292,11 @@ CodeMirror.defineMode("sass", function(config) {
       dedent(state);
     }
 
-    if (style === "atom" && htmlTag(current)){
+    if (style === "atom"){
       indent(state);
     }
 
-    if (style !== "error"){
+    if (style !== null){
       var startOfToken = stream.pos - current.length;
       var withCurrentIndent = startOfToken + (config.indentUnit * state.indentCount);
 
@@ -312,7 +306,7 @@ CodeMirror.defineMode("sass", function(config) {
         var scope = state.scopes[i];
 
         if (scope.offset <= withCurrentIndent){
-      newScopes.push(scope);
+          newScopes.push(scope);
         }
       }
 
@@ -347,3 +341,5 @@ CodeMirror.defineMode("sass", function(config) {
 });
 
 CodeMirror.defineMIME("text/x-sass", "sass");
+
+});
