@@ -12,6 +12,54 @@ $(function() {
 			e.preventDefault();
 		}
 	});
+	
+
+	var showHelp = function(cm, showDocTab) {
+		var selection = cm.getSelection();
+		var topic;
+		if(selection) {
+			topic = selection;
+		} else {
+			// Get cursor
+			var cursor = cm.getCursor();
+
+			//WORKAROUND: Resolve getTokenAt bug by adding 1 to column
+			var c = {
+				line: cursor.line,
+				ch: cursor.ch + 1
+			};
+			
+			// Search for token under the cursor
+			var token = cm.getTokenAt(c);
+			if(token.string) {
+				topic = $.trim(token.string);
+			} else {
+				topic = '';
+			}
+		}
+		
+		$.post("/help", {
+			topic  : topic, 
+			style : $("#pod_style_selector", $('#options-dialog')).val()
+			}, 
+			function(html) {
+				// Populate documentation tab
+				var $iframe = $("#doc");
+				var scrollTop = $iframe.contents().scrollTop();
+				var doc = $iframe[0].contentDocument;
+				doc.open();
+				doc.write(html);
+				doc.close();
+				$iframe.contents().scrollTop(scrollTop);
+
+				if(showDocTab) {
+					// Show documentation tab
+					$('a[href="#doc-tab"]').click();
+				}
+			}
+		);
+	};
+
 
 	// Editors hash object
 	var editors = {};
@@ -55,6 +103,8 @@ $(function() {
 		// Refresh...
 		editor.removeOverlay(window.whitespaceOverlay);
 		editor.addOverlay(window.whitespaceOverlay);
+		diffEditor.removeOverlay(window.whitespaceOverlay);
+		diffEditor.addOverlay(window.whitespaceOverlay);
 	};
 
 	// Create the readonly CodeMirror input editor instance
@@ -81,11 +131,11 @@ $(function() {
 
 	var diffEditor = CodeMirror.fromTextArea(document.getElementById('diff'), {
 		//TODO these should be configurable!
-		
 		tabSize: 4,
 		indentUnit: 4,
 		indentWithTabs: true,
-		readOnly: true
+		readOnly: true,
+		flattenSpans: false
 	});
 
 	// Some friendly advice
@@ -157,21 +207,6 @@ $(function() {
 	}, 5000);
  
 
-	function displayHelp(cm) {
-		var selection = cm.getSelection();
-		if(selection) {
-			_displayHelp(selection, true);
-		} else {
-			// Search for token under the cursor
-			var token = cm.getTokenAt(cm.getCursor());
-			if(token.string) {
-				_displayHelp($.trim(token.string), true);
-			} else {
-				_displayHelp('', true);
-			}
-		}
-	}
-
 	// Show a '*' after the tab name if it is modified	
 	var showStarAfterNameIfModified = function(editor) {
 		if(editor.isClean()) {
@@ -287,9 +322,9 @@ $(function() {
 			styleActiveLine: true,
 			flattenSpans: false,
 			extraKeys: {
-				//"F1": function(cm) {
-				//	displayHelp(cm);
-				//},
+				"F1": function(cm) {
+					showHelp(cm, true);
+				},
 				'Ctrl-Alt-F': function(cm) {
 					$(document).trigger('action-ack');
 				},
@@ -377,6 +412,7 @@ $(function() {
 						if($("#preview-tab").is(":visible")) {
 							$('a[href="#preview-tab"]').click();
 						}
+
 					} else if(mode == "markdown") {
 						if($("#preview-tab").is(":visible")) {
 							$('a[href="#preview-tab"]').click();
@@ -391,7 +427,7 @@ $(function() {
 
 				showStarAfterNameIfModified(cm);
 		};
-
+		
 		editor.on("change", onChange);
 		editor.on("focus", onChange);
 
@@ -473,32 +509,6 @@ $(function() {
 		return false;
 	});
 	
-	function _displayHelp(topic, bShowDialog) {
-		$.post('/help_search', {"topic": topic}, function(results) {
-			if(results.length > 0) {
-				$(".topic").val(topic);
-				var html = '';
-				for(var i = 0; i < results.length; i++) {
-					html += '<option value="' + i + '">' + results[i].podname + "  (" + results[i].context + ")" +'</option>';
-				}
-
-				$(".results")
-					.html(html)
-					.change(function() {
-						var index = $(':selected', this).val();
-						$(".preview").html(results[index].html);
-					}).change().show();
-			} else {
-				$(".topic").val(topic);
-				$(".results").html('').hide();
-				$(".preview").html('<span class="text-error">No results found!</span>');
-			}
-			if(bShowDialog) {
-				$(".topic").select().focus();
-			}
-		});
-	}
-
 	window.ChangeMode = function(cm, modeFile, mode) {
 		if(typeof mode == 'undefined') {
 			mode = modeFile;
@@ -835,16 +845,6 @@ $(function() {
 	// to the current editor
 	$(".modal").hide().on('hidden', onCloseFocusEditor);
 
-	$(".topic").typeahead({
-		source : function(query, process) {
-			$.post('/typeahead', {'query': query}, function(matches) {
-				process(matches);
-			});
-		}
-	}).change(function() {
-		_displayHelp($(this).val(), true);
-	});
-	
 	$(document).on('action-new-file', function() {
 		AddEditorTab();
 	});
@@ -967,8 +967,8 @@ $(function() {
 	});
 	
 
-	$(document).on('action-git-diff', function() {
-		$.post('/git_diff', {}, function(result) {
+	var git = function(cmd) {
+		$.post('/git', {cmd: cmd}, function(result) {
 
 			diffEditor.setValue('');
 
@@ -985,6 +985,18 @@ $(function() {
 
 			diffEditor.refresh();
 		});
+	};
+
+	$(document).on('action-git-diff', function() {
+		git('diff');
+	});
+
+	$(document).on('action-git-log', function() {
+		git('log');
+	});
+
+	$(document).on('action-git-status', function() {
+		git('status');
 	});
 
 	$(document).on('action-perl-strip', function() {
@@ -1121,70 +1133,53 @@ $(function() {
 		});
 	});
 
+	var showCmdOutput = function(result) {
+
+		var outputEditor = GetOutputEditor();
+
+		outputEditor.setValue('');
+
+		// Handle STDERR
+		if (result.stderr.length) {
+			outputEditor.setValue(result.stderr + "\n");
+		}
+
+		// Handle STDOUT
+		outputEditor.setValue(outputEditor.getValue() + result.stdout);
+
+		// Show output tab
+		$('a[href="#output-tab"]').tab('show');
+
+		outputEditor.refresh();
+	};
+
 	$(document).on('action-midgen', function() {
-		$.post('/midgen', {}, function(result) {
-
-			var outputEditor = GetOutputEditor();
-
-			outputEditor.setValue('');
-
-			// Handle STDERR
-			if (result.stderr.length) {
-				outputEditor.setValue(result.stderr + "\n");
-			}
-
-			// Handle STDOUT
-			outputEditor.setValue(outputEditor.getValue() + result.stdout);
-
-			// Show output tab
-			$('a[href="#output-tab"]').tab('show');
-
-			outputEditor.refresh();
-		});
+		$.post('/midgen', {}, showCmdOutput);
 	});
 
-	$(document).on('action-minil-test', function() {
-		$.post('/minil_test', {}, function(result) {
-
-			var outputEditor = GetOutputEditor();
-
-			outputEditor.setValue('');
-
-			// Handle STDERR
-			if (result.stderr.length) {
-				outputEditor.setValue(result.stderr + "\n");
-			}
-
-			// Handle STDOUT
-			outputEditor.setValue(outputEditor.getValue() + result.stdout);
-
-			// Show output tab
-			$('a[href="#output-tab"]').tab('show');
-
-			outputEditor.refresh();
-		});
+	$(document).on('action-build', function() {
+		$.post('/project', {cmd: 'build'}, showCmdOutput);
 	});
 
-	$(document).on('action-dzil-test', function() {
-		$.post('/dzil_test', {}, function(result) {
+	$(document).on('action-build-test', function() {
+		$.post('/project', {cmd: 'test'}, showCmdOutput);
+	});
 
-			var outputEditor = GetOutputEditor();
+	$(document).on('action-build-clean', function() {
+		$.post('/project', {cmd: 'clean'}, showCmdOutput);
+	});
 
-			outputEditor.setValue('');
+	$(document).on('action-cpanm', function() {
+		var editor = GetCurrentEditor();
+		var selection = editor.getSelection();
+		if(!selection) {
+			setTimeout(function() {
+				alert("Please select some text to search for it in your current project folder");
+			}, 0);
+			return;
+		}
 
-			// Handle STDERR
-			if (result.stderr.length) {
-				outputEditor.setValue(result.stderr + "\n");
-			}
-
-			// Handle STDOUT
-			outputEditor.setValue(outputEditor.getValue() + result.stdout);
-
-			// Show output tab
-			$('a[href="#output-tab"]').tab('show');
-
-			outputEditor.refresh();
-		});
+		$.post('/cpanm', {module: selection}, showCmdOutput);
 	});
 
 });
