@@ -10,9 +10,9 @@ $(function() {
 		// Get the action name
 		var dirs = $("#dirs", $dialog).val();
 		
-		// find duplicate perl code
-		window.sendMessage("find-duplicate-perl-code", { "dirs": dirs }, function(result) {
-			if(result.error == '') {
+		// Find duplicate Perl code via Code::CutNPaste
+		$.post("/code_cutnpaste", { "dirs": dirs }, function(result) {
+			if(result.error === '') {
 				if(result.count > 0) {
 					// Show it in the output pane
 					$("#output").val(result.output);
@@ -34,7 +34,7 @@ $(function() {
 	});
 
 	// Open the find duplicate perl dialog when the action is triggered
-	$(document).on('action-find-duplicate-perl-code', function() {
+	$(document).on('action-code-cutnpaste', function() {
 		$dialog.modal('show');
 		$dialog.on('shown', function() {
 			$("#dirs").val('lib').focus();
