@@ -27,8 +27,7 @@ $(function() {
 	});
 	
 	$("#theme_selector", $optionsDialog).change(function() {
-		var $selectedTheme = $(":selected", this);
-		var theme = $selectedTheme.val();
+		var theme = $(":selected", this).val();
 
 		if (theme != "default") {
 			// Load theme CSS dynamically if it is not default
@@ -43,7 +42,11 @@ $(function() {
 		var editor = GetCurrentEditor();
 		editor.setOption("theme", theme);
 	});
-	
+
+	$("#pod_style_selector", $optionsDialog).change(function() {
+		$('a[href="#preview-tab"]').click();
+	});
+
 	$("#line_numbers_checkbox", $optionsDialog).change(function() {
 		var editor = GetCurrentEditor();
 		editor.setOption('lineNumbers', $(this).is(':checked'));
