@@ -13,8 +13,8 @@ $(function() {
 
 	var perlCritic = function() {
 		var editor = GetCurrentEditor();
-		window.sendMessage(
-			'perl-critic', 
+		$.post(
+			'perl_critic', 
 			{
 				"source": editor.getValue(), 
 				"severity": $(':selected', '#critic_severity_selector').val()
@@ -46,6 +46,10 @@ $(function() {
 					
 					var id = 0;
 
+					var showTab = function() {
+						$('a[href="#problems-tab"]').tab('show');
+					};
+
 					for(i = 0; i < problems.length; i++) {
 						var problem = problems[i];
 
@@ -57,16 +61,13 @@ $(function() {
 						msg.appendChild(document.createTextNode(problem.description));
 						msg.className = "farabi-error";
 
-						
 						widgets.push({
 							problem      : problem,
 							widget       : editor.addLineWidget(problem.line_number - 1, msg, {coverGutter: true, noHScroll: true}),
-							id           : id,
+							id           : id
 						});
 
-						$(msg).click(function() {
-							$('a[href="#problems-tab"]').tab('show');
-						});
+						$(msg).click(showTab);
 
 						html += 
 							'<tr id="' + id + '">' +
@@ -88,22 +89,17 @@ $(function() {
 
 				$("#problems > table").html(html);
 				$("#problems>  table > tbody > tr").click(function() { 
-					var line = parseInt($(".problem-line", this).text());
+					var line = parseInt($(".problem-line", this).text(), 10);
 					var editor = GetCurrentEditor();
 					editor.setCursor({line: line - 1, ch: 0});
-					
-					console.error($(this));
-					console.error($(this).html());
-					
+
 					var problem;
-					console.error("id = " + $(this).attr("id"));
 					for(var i = 0; i < widgets.length; i++) {
 						if(widgets[i].id == $(this).attr("id")) {
 							problem  = widgets[i].problem;
 							break;
 						}
 					} 
-					console.error(problem);
 					
 					if(!problem) {
 						return;
