@@ -15,13 +15,13 @@ $(function() {
 	});
 
 	var evalCommand = function() {
-		window.sendMessage("repl-eval", { 
+		$.post("/repl_eval", { 
 				"command"    : $("#command", $replDialog).val(),
-				"runtime" : $("#runtime :selected", $replDialog).attr("id"),
+				"runtime" : $("#runtime :selected", $replDialog).attr("id")
 			},
 			function(result) {
 				var oldOutput = $("#output", $replDialog).val();
-				if(result.err != '') {
+				if(result.err !== '') {
 					result.out += "\nError: " + result.err;
 				}
 				
