@@ -22,14 +22,6 @@ $(function() {
 	});
 
 	var runtimeHelp = {
-		"rakudo"    : {
-			text    : "<b>Rakudo</b> Perl 6, or simply Rakudo, is a compiler for the Perl 6 programming language. It runs on the Parrot virtual machine.",
-			url     : "http://rakudo.org/about"
-		},
-		"parrot"    : {
-			text   : "<b>Parrot</b> is a virtual machine designed to efficiently compile and execute bytecode for dynamic languages.",
-			url    : "http://www.parrot.org/"
-		},
 		"perl": {
 			text    : "<b>Perl 5</b> is a highly capable, feature-rich programming language with over 24 years of development.",
 			url     : "http://perl.org"
@@ -71,22 +63,12 @@ $(function() {
 			}
 		} else if(runtime == "perl") { 
 			// Perl
-			window.sendMessage('run-perl', {"source": editor.getValue(), "input": inputEditor.getValue() }, function(result) {
-				show_cmd_output(result);
-			});
-		} else if(runtime == "rakudo") {
-			// Rakudo Perl 6
-			window.sendMessage('run-rakudo', {"source": editor.getValue(), "input": inputEditor.getValue() }, function(result) {
-				show_cmd_output(result);
-			});
-		} else if(runtime == "parrot") {
-			// Rakudo Perl 6
-			window.sendMessage('run-parrot', {"source": editor.getValue(), "input": inputEditor.getValue() }, function(result) {
+			$.post('/run_perl', {"source": editor.getValue(), "input": inputEditor.getValue() }, function(result) {
 				show_cmd_output(result);
 			});
 		} else if(runtime == "perlbrew-exec") {
 			// PerlBrew exec
-			window.sendMessage('run-perlbrew-exec', {"source": editor.getValue(), "input": inputEditor.getValue() }, function(result) {
+			$.post('/run_perlbrew_exec', {"source": editor.getValue(), "input": inputEditor.getValue() }, function(result) {
 				show_cmd_output(result);
 			});
 		}
